//**********************************************************************
//**********************************************************************
//
//                            Dungeon Drawing Code
//
//**********************************************************************
//**********************************************************************

#include <vectrex.h>
#include "controller.h"
#include "string.h"

#include "aklabeth.h"

const unsigned int const xWidthHalf[] =
{
	(MAX_WIDTH/1)/2, // level 1
	(MAX_WIDTH/2)/2, // level 2
	(MAX_WIDTH/3)/2, // level 3
	(MAX_WIDTH/4)/2, // level 4
	(MAX_WIDTH/5)/2, // level 5
	(MAX_WIDTH/6)/2, // level 6
	(MAX_WIDTH/7)/2, // level 7
	(MAX_WIDTH/8)/2, // level 8
	(MAX_WIDTH/9)/2 // level 9
};
const unsigned int const  yHeightHalf[] =
{
	(MAX_HEIGHT/1)/2, // level 1
	(MAX_HEIGHT/2)/2, // level 2
	(MAX_HEIGHT/3)/2, // level 3
	(MAX_HEIGHT/4)/2, // level 4
	(MAX_HEIGHT/5)/2, // level 5
	(MAX_HEIGHT/6)/2, // level 6
	(MAX_HEIGHT/7)/2, // level 7
	(MAX_HEIGHT/8)/2, // level 8
	(MAX_HEIGHT/9)/2 // level 9
};

#define _DDRAWCalcRect(r,Level) \
(r)->left =  -xWidthHalf[Level];  \
(r)->right = +xWidthHalf[Level]; \
(r)->top =   +yHeightHalf[Level]; \
(r)->bottom= -yHeightHalf[Level];




const signed const int s_dscale[] =
{
	//        0xff,
	0x80,
	0x40,
	0x20,
	0x10,
	0x08,
	0x04,
	0x02,
	0x01,
	0x00,
	0x00,
	0x00,
	0x00,
};

//**********************************************************************
//
//                            Draw the dungeon
//
//**********************************************************************
#define START_UPPER_LEFT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(*y_depth, -*x_depth);\
	VIA_t1_cnt_lo = *dscale;\
} while (0)
#define START_UPPER_RIGHT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(*y_depth, *x_depth);\
	VIA_t1_cnt_lo = *dscale;\
} while (0)
#define START_LOWER_LEFT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(-*y_depth, -*x_depth);\
	VIA_t1_cnt_lo = *dscale;\
} while (0)
#define START_LOWER_RIGHT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(-*y_depth, *x_depth);\
	VIA_t1_cnt_lo = *dscale;\
} while (0)

#define START_UPPER_LEFT_NEXT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(*(y_depth+1), -*(x_depth+1));\
	VIA_t1_cnt_lo = *(dscale+1);\
} while (0)
#define START_UPPER_RIGHT_NEXT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(*(y_depth+1), *(x_depth+1));\
	VIA_t1_cnt_lo = *(dscale+1);\
} while (0)
#define START_LOWER_LEFT_NEXT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(-*(y_depth+1), -*(x_depth+1));\
	VIA_t1_cnt_lo = *(dscale+1);\
} while (0)
#define START_LOWER_RIGHT_NEXT() do {\
	VIA_t1_cnt_lo = 0x80;\
	Reset0Ref();\
	Moveto_d(-*(y_depth+1), *(x_depth+1));\
	VIA_t1_cnt_lo = *(dscale+1);\
} while (0)
const signed char WallBackFull[]=
{    (signed char) 0x00, +0x12, -0x26,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x4B,  // pattern, y, x
	(signed char) 0xFF, -0x25, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x4B,  // pattern, y, x
	(signed char) 0xFF, +0x25, +0x00,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char Pit[]=
{    (signed char) 0x00, -0x16, -0x1E,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x3B,  // pattern, y, x
	(signed char) 0xFF, -0x0B, +0x0F,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x59,  // pattern, y, x
	(signed char) 0xFF, +0x0B, +0x0F,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char SceneList_3[]=
{    (signed char) 0x00, -0x1B, -0x15,  // pattern, y, x
	(signed char) 0xFF, +0x35, +0x00,  // pattern, y, x
	(signed char) 0x00, -0x08, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x29,  // pattern, y, x
	(signed char) 0x00, +0x08, +0x00,  // pattern, y, x
	(signed char) 0xFF, -0x35, +0x00,  // pattern, y, x
	(signed char) 0x00, +0x06, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x29,  // pattern, y, x
	(signed char) 0x00, +0x0D, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x29,  // pattern, y, x
	(signed char) 0x00, +0x0D, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x29,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char WallSideLeftOpen[]=
{    (signed char) 0x00, -0x13, -0x26,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x26,  // pattern, y, x
	(signed char) 0x00, -0x12, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x49, +0x00,  // pattern, y, x
	(signed char) 0x00, -0x12, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x26,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char WallSideRightOpen[]=
{    (signed char) 0x00, -0x13, +0x25,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x26,  // pattern, y, x
	(signed char) 0x00, -0x12, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x49, +0x00,  // pattern, y, x
	(signed char) 0x00, -0x12, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x26,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char RightDoor[]=
{    (signed char) 0x00, -0x18, +0x32,  // pattern, y, x
	(signed char) 0xFF, +0x1D, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x0C,  // pattern, y, x
	(signed char) 0xFF, -0x23, +0x00,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char LeftDoor[]=
{    (signed char) 0x00, -0x1F, -0x40,  // pattern, y, x
	(signed char) 0xFF, +0x24, +0x00,  // pattern, y, x
	(signed char) 0xFF, -0x01, +0x0C,  // pattern, y, x
	(signed char) 0xFF, -0x1D, +0x00,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char CenterDoor[]=
{    (signed char) 0x00, -0x13, -0x0C,  // pattern, y, x
	(signed char) 0xFF, +0x16, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x17,  // pattern, y, x
	(signed char) 0xFF, -0x16, +0x00,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char WallLeftDiagonals[]=
{    (signed char) 0x00, -0x25, -0x4C,  // pattern, y, x
	(signed char) 0xFF, +0x12, +0x26,  // pattern, y, x
	(signed char) 0x00, +0x25, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x12, -0x26,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char WallRightDiagonals[]=
{    (signed char) 0x00, -0x25, +0x4B,  // pattern, y, x
	(signed char) 0xFF, +0x12, -0x26,  // pattern, y, x
	(signed char) 0x00, +0x25, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x12, +0x26,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char WallBackOpen[]=
{    (signed char) 0x00, +0x12, -0x26,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x4B,  // pattern, y, x
	(signed char) 0x00, -0x25, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x4B,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char LadderUp[]=
{    (signed char) 0x00, -0x08, -0x15,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x29,  // pattern, y, x
	(signed char) 0x00, +0x0D, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x29,  // pattern, y, x
	(signed char) 0x00, +0x0D, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x29,  // pattern, y, x
	(signed char) 0x00, +0x08, +0x00,  // pattern, y, x
	(signed char) 0xFF, -0x35, +0x00,  // pattern, y, x
	(signed char) 0x00, +0x06, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x29,  // pattern, y, x
	(signed char) 0x00, -0x06, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x35, +0x00,  // pattern, y, x
	(signed char) 0x00, +0x06, -0x18,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x59,  // pattern, y, x
	(signed char) 0xFF, -0x0B, -0x0F,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x3B,  // pattern, y, x
	(signed char) 0xFF, +0x0B, -0x0F,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char LadderDown[]=
{    (signed char) 0x00, -0x1B, -0x15,  // pattern, y, x
	(signed char) 0xFF, +0x35, +0x00,  // pattern, y, x
	(signed char) 0x00, -0x08, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x29,  // pattern, y, x
	(signed char) 0x00, +0x08, +0x00,  // pattern, y, x
	(signed char) 0xFF, -0x35, +0x00,  // pattern, y, x
	(signed char) 0x00, +0x06, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x29,  // pattern, y, x
	(signed char) 0x00, -0x01, -0x09,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x3B,  // pattern, y, x
	(signed char) 0xFF, -0x0B, +0x0F,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x59,  // pattern, y, x
	(signed char) 0xFF, +0x0B, +0x0F,  // pattern, y, x
	(signed char) 0x00, +0x0E, +0x09,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x29,  // pattern, y, x
	(signed char) 0x00, +0x0D, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x29,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char WallBackRightOpen[]=
{    (signed char) 0x00, -0x13, +0x25,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x4B,  // pattern, y, x
	(signed char) 0xFF, +0x25, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x4B,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};
const signed char WallBackLeftOpen[]=
{    (signed char) 0x00, +0x12, -0x26,  // pattern, y, x
	(signed char) 0xFF, +0x00, +0x4B,  // pattern, y, x
	(signed char) 0xFF, -0x25, +0x00,  // pattern, y, x
	(signed char) 0xFF, +0x00, -0x4B,  // pattern, y, x
	(signed char) 0x01 // endmarker (high bit in pattern not set)
};

#define DRAW(list) \
dp_VIA_cntl = 0xce; \
Draw_VLp_lessGap(list); \
Reset0Ref();
//	dp_VIA_cntl = (unsigned int)0xcc;            // enable zero, enable all blank


//Draw_VLp_lessGap(list); \

void _DRAWDungeon(int Left,int Centre,int Right, int Room,int Monster, int distance)
{
	//    int x1,y1,x,y,y2;
	//    RECT r;
	
	// draw from outer to inner
	if (ISDRAWOPEN(Left))                    // Do we draw the left edge
	{
		DRAW(WallSideLeftOpen);
	}
	else                                    // If closed, draw left diags
	{
		DRAW(WallLeftDiagonals);
	}
	
	if (ISDRAWOPEN(Right))                    // Do we draw the right edge
	{
		DRAW(WallSideRightOpen);
	}
	else                                    // If closed draw right diags
	{
		DRAW(WallRightDiagonals);
	}
	
	// draw inner
	if (!ISDRAWOPEN(Centre))                // Back wall ?
	{
		if ((!ISDRAWOPEN(Left)) && (!ISDRAWOPEN(Right)))
		{
			// draw a square
			DRAW(WallBackFull);
		}
		else  if (!ISDRAWOPEN(Left))
		{
			// draw three sides right open
			DRAW(WallBackRightOpen);
		}
		else  if (!ISDRAWOPEN(Right))
		{
			// draw three sides Left open
			DRAW(WallBackLeftOpen);
		}
		else
		{
			// draw bottom and top
			DRAW(WallBackOpen);
		}
	}
	if (Left == DT_DOOR)
	{
		DRAW(LeftDoor);
	}
	if (Right == DT_DOOR)
	{
		DRAW(RightDoor);
	}
	if (Centre == DT_DOOR)
	{
		DRAW(CenterDoor);
	}
	
	if (Room == DT_LADDERUP)
	{
		DRAW(LadderUp);
	}
	else if (Room == DT_LADDERDN)
	{
		DRAW(LadderDown);
	}
	else if (Room == DT_PIT)
	{
		DRAW(Pit);
	}
	else if (Room == DT_GOLD)                // Draw the gold (as a mimic)
	{
		//        DRAWMonster((r.left+r.right)/2,r.bottom,MN_MIMIC,distance);
		DRAWMonster(0,0,MN_MIMIC,distance);
		Reset0Ref();
//	dp_VIA_cntl = (unsigned int)0xcc;            // enable zero, enable all blank
	}
	
	
	if (Monster > 0)                    // Monster here ?
	{
		//        DRAWMonster((r.left+r.right)/2,r.bottom,Monster,distance);
		DRAWMonster(0,0,Monster,distance);
		Reset0Ref();
//	dp_VIA_cntl = (unsigned int)0xcc;            // enable zero, enable all blank
	}
	
}

unsigned int *dscale;




//RECT rOut,rIn;
COORD /*Dir*/Pos,Next;
void DDRAWDraw(PLAYER *p,DUNGEONMAP *d)
{
	dscale = s_dscale;
	int monsterDone = 0;
	int x,y;
	unsigned int  Level = 0;
	int Monster,Front,Left,Right;
	Pos = p->Dungeon;                        // Get position
	x= p->DungDir.x;
	y= p->DungDir.y;
	Reset0Ref();

       
    if (y == 0)
    {
		Next.y = Pos.y;
	do
	{
		dp_VIA_t1_cnt_lo = *dscale;
		Level++;                            // Next level
		Next.x = Pos.x +x;        // Next position
		Front = d->Map[Next.x][Next.y];        // What's in front ?
		
		Left = d->Map[Pos.x][Pos.y-x];
		Right = d->Map[Pos.x][Pos.y+x];
		if (monsterDone == 0)
		{
			Monster = DDRAWFindMonster(d,&Pos);    // Find ID of monster here
			if (Monster >= 0)                    // Find Type if Found
			{
				Monster = d->Monster[Monster].Type;
			monsterDone =1;
			}
		}
		else
		Monster = -1;
		_DRAWDungeon(Left,Front,Right, d->Map[Pos.x][Pos.y],Monster,Level);               // Draw the dungeon
		
		Pos = Next;                            // Next position down
		dscale++;
		
	}
	while (Level < MAX_VIEW_DEPTH && ISDRAWOPEN(Front));
	
            }
    else
    {
		Next.x = Pos.x;
	do
	{
		dp_VIA_t1_cnt_lo = *dscale;
		Level++;                            // Next level
		Next.y = Pos.y +y;
		Front = d->Map[Next.x][Next.y];        // What's in front ?
		
			Left = d->Map[Pos.x+y][Pos.y];
			Right = d->Map[Pos.x-y][Pos.y];
		if (monsterDone == 0)
		{
			Monster = DDRAWFindMonster(d,&Pos);    // Find ID of monster here
			if (Monster >= 0)                    // Find Type if Found
			{
				Monster = d->Monster[Monster].Type;
			monsterDone =1;
			}
		}
		else
		Monster = -1;
		_DRAWDungeon(Left,Front,Right, d->Map[Pos.x][Pos.y],Monster,Level);               // Draw the dungeon
		
		Pos = Next;                            // Next position down
		dscale++;
		
	}
	while (Level < MAX_VIEW_DEPTH && ISDRAWOPEN(Front));
    }
    /*
	do
	{
		dp_VIA_t1_cnt_lo = *dscale;
		Level++;                            // Next level
		Next.x = Pos.x +x;        // Next position
		Next.y = Pos.y +y;
		Front = d->Map[Next.x][Next.y];        // What's in front ?
		
		if (y == 0)
		{
			Left = d->Map[Pos.x][Pos.y-x];
			Right = d->Map[Pos.x][Pos.y+x];
			
		}
		else
		{
			Left = d->Map[Pos.x+y][Pos.y];
			Right = d->Map[Pos.x-y][Pos.y];
		}
		if (monsterDone == 0)
		{
			Monster = DDRAWFindMonster(d,&Pos);    // Find ID of monster here
			if (Monster >= 0)                    // Find Type if Found
			{
				Monster = d->Monster[Monster].Type;
			monsterDone =1;
			}
		}
		else
		Monster = -1;
		_DRAWDungeon(Left,Front,Right, d->Map[Pos.x][Pos.y],Monster,Level);               // Draw the dungeon
		
		Pos = Next;                            // Next position down
		dscale++;
		
	}
	while (Level < MAX_VIEW_DEPTH && ISDRAWOPEN(Front));
*/	
	
	if (monsterDone == 0)
    {
        	// check for one more monster in a "doorway"
        	if (! ISDRAWOPEN(Front))
        	{
        		Monster = DDRAWFindMonster(d,&Pos);    // Find ID of monster here
        		if (Monster >= 0)                    // Find Type if Found
        		{
        			Level++;                            // Next level
        			//            _DDRAWCalcRect(&rIn,Level);
        			Monster = d->Monster[Monster].Type;
        			/*
        			RECT r;
        			DRAWSetRect(&r,                           // Get the object area
        			(rIn.left+rOut.left)/2,
        			(rIn.top+rOut.top)/2,
        			(rIn.right+rOut.right)/2,
        			(rIn.bottom+rOut.bottom)/2);
        			*/
        			
        			DRAWMonster(0,0,Monster,Level);
        			
        		}
        		
        		
        		
        	}
    }
	
	
}

//**********************************************************************
//
//                    Calculate display rectangle
//
//**********************************************************************




//**********************************************************************
//
//                    Find Monster ID at given location
//
//**********************************************************************

int DDRAWFindMonster(DUNGEONMAP *d,COORD *c)
{
	int i,n = -1;
	for (i = 0;i < d->MonstCount;i++)
	if (c->x == d->Monster[i].Loc.x && c->y == d->Monster[i].Loc.y && d->Monster[i].Alive != 0)
	n = i;
	return n;
}
