 .module printSync.s
 .area .text

;***************************************************************************
; DEFINE SECTION
;***************************************************************************
; load vectrex bios routine definitions
music1          =     0xFD0D
VIA_port_b      =     0xD000   ;VIA port B data I/O register
VIA_port_a      =     0xD001   ;VIA port A data I/O register (handshaking)
VIA_t1_cnt_lo   =     0xD004   ;VIA timer 1 count register lo (scale factor)
VIA_t1_cnt_hi   =     0xD005   ;VIA timer 1 count register hi
VIA_shift_reg   =     0xD00A   ;VIA shift register
VIA_cntl        =     0xD00C   ;VIA control register
VIA_int_flags   =     0xD00D   ;VIA interrupt flags register

Intensity_5F    =     0xF2A5   ;
Wait_Recal      =     0xF192   ;
Mov_Draw_VLc_a  =     0xF3AD   ;count y x y x ...
Moveto_d        =     0xF312   ;
;***************************************************************************
; Variable / RAM SECTION
;***************************************************************************
; insert your variables (RAM usage) in the BSS section
; user RAM starts at 0xc880 
                    .area .bss      
; Warning - org line found, my be countering relocatable code!
;                    ORG      0xc880                        ; start of our ram space 
;***************************************************************************
; HEADER SECTION
;***************************************************************************
; The cartridge ROM starts at address 0
                    .area .text     
; Warning - org line found, my be countering relocatable code!
;                    ORG      0 

;ZERO ing the integrators takes time. Measures at my vectrex show e.g.:
;If you move the beam with a to x = -127 and y = -127 at diffferent scale values, the time to reach zero:
;- scale 0xff -> zero 110 cycles
;- scale 0x7f -> zero 75 cycles
;- scale 0x40 -> zero 57 cycles
;- scale 0x20 -> zero 53 cycles
ZERO_DELAY          =      1                            ; delay 7 counter is exactly 111 cycles delay between zero SETTING and zero unsetting (in moveto_d) 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;U = address of vectorlist
;X = (y,x) position of vectorlist (this will be point 0,0), positioning on screen
;A = scalefactor "Move" (after sync)
;B = scalefactor "Vector" (vectors in vectorlist)
;
;     mode, rel y, rel x,                                             
;     mode, rel y, rel x,                                             
;     .      .      .                                                
;     .      .      .                                                
;     mode, rel y, rel x,                                             
;     0x02
; where mode has the following meaning:         
; negative draw line                    
; 0 move to specified endpoint                              
; 1 sync (and move to list start and than to place in vectorlist)      
; 2 end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 .setdp 0xd000,_DATA

 .globl _draw_synced_list
 .globl draw_synced_list
_draw_synced_list: 
draw_synced_list: 
 pshs u

 lda 2+2,s ; calling from C
 ldu 3+2,s ; than parameters are on stack
                    pshs     a                            ; remember out different scale factors 
                    pshs     b 
                                                          ; first list entry (first will be a sync + moveto_d, so we just stay here!) 
                    lda      ,u+                          ; this will be a "1" 
 .globl sync
sync: 
                    deca                                  ; test if real sync - or end of list (2) 
                    bne      drawdone                     ; if end of list -> jump 
; zero integrators
                    ldb      #0xCC                         ; zero the integrators 
                    stb      *VIA_cntl                    ; store zeroing values to cntl 
                    ldb      #ZERO_DELAY                  ; and wait for zeroing to be actually done 
; reset integrators
                    clr      *VIA_port_a                  ; reset integrator offset 
                    lda      #0b10000010 
; wait that zeroing surely has the desired effect!
 .globl zeroLoop
zeroLoop: 
                    sta      *VIA_port_b                  ; while waiting, zero offsets 
                    decb     
                    bne      zeroLoop 
                    inc      *VIA_port_b 
; unzero is done by moveto_d
                    lda      1,s                          ; scalefactor move 
                    sta      *VIA_t1_cnt_lo               ; to timer t1 (lo= 
                    tfr      x,d                          ; load our coordinates of "entry" of vectorlist 
                    jsr      Moveto_d                     ; move there 
                    lda      ,s                           ; scale factor vector 
                    sta      *VIA_t1_cnt_lo               ; to timer T1 (lo) 
 .globl moveTo
moveTo: 
                    ldd      ,u++                         ; do our "internal" moveto d 
                    beq      nextListEntry                ; there was a move 0,0, if so 
                    jsr      Moveto_d 
 .globl nextListEntry
nextListEntry: 
                    lda      ,u+                          ; load next "mode" byte 
                    beq      moveTo                       ; if 0, than we should move somewhere 
                    bpl      sync                         ; if still positive it is a 1 pr 2 _> goto sync 
; now we should draw a vector 
                    ldd      ,u++                         ;Get next coordinate pair 
                    STA      *VIA_port_a                  ;Send Y to A/D 
                    CLR      *VIA_port_b                  ;Enable mux 
                    LDA      #0xff                         ;Get pattern byte 
                    INC      *VIA_port_b                  ;Disable mux 
                    STB      *VIA_port_a                  ;Send X to A/D 
                    LDB      #0x40                         ;B-reg = T1 interrupt bit 
                    CLR      *VIA_t1_cnt_hi               ;Clear T1H 
                    STA      *VIA_shift_reg               ;Store pattern in shift register 
 .globl setPatternLoop
setPatternLoop: 
                    BITB     *VIA_int_flags               ;Wait for T1 to time out 
                    beq      setPatternLoop               ; wait till line is finished 
                    CLR      *VIA_shift_reg               ; switch the light off (for sure) 
                    bra      nextListEntry 

 .globl drawdone
drawdone: 
                    puls     d,u                            ; correct stack and go back 
                    rts      
;***************************************************************************
; DATA SECTION
;***************************************************************************



